/* 
** shell for testing compression programs.
** BYTE, 11/90
** S. Apiki
** Use with huff.asm and lzw.asm, discussed in the March, 1991 issue of BYTE.
**
** To build the executable with Turbo:
** tcc -c -mC cshell.c
** tcc -c -mC eshell.c
** tcc -c -mC csubs.c
** tasm huff.asm
** tasm lzw.asm
** tlink c0c cshell csubs lzw huff, crumple.exe,, emu.lib mathc.lib cc.lib
** tlink c0c eshell csubs lzw huff, uncrumple.exe,, emu.lib mathc.lib cc.lib
** These steps will build two files, crumple.exe and uncrumpl.exe, the
** compressor and decompressor.
*/

#include <stdio.h>
#include <fcntl.h>
#include <io.h>
#include <errno.h>
#include <string.h>
#include <sys/stat.h>
#include <stdlib.h>

#include "crumple.h"

/* support files are in csubs.c */
void Error(int type, int isfatal, char * string);	/* error handler */
int open_outfile(char* filespec);

/*
* Main program begins here.
*
* Shell takes an input file, an output file, and a duplicate file name.
*
*/

void main(int argc, char** argv)
{

int infh, outfh, dupfh;
unsigned long Length, Outlen;

	/* open input file */
	if(argc < 3)
		Error(C_USAGE, FATAL, NULL);
	infh=open(argv[1], O_BINARY|O_RDONLY);
	if(infh == -1)
		Error(FILEOPEN, FATAL, argv[1]);

	/* open output file */
	outfh=open_outfile(argv[2]);
	if (outfh == -1){	
		close(infh);
		Error(CANTCREATE, FATAL, argv[2]);
		}

	if(argc >3){
		switch(toupper(*(argv[3]+1))){
			case 'L':
				write( outfh, "cf\x1al", 4);
				Outlen = lzw_compress(infh, outfh);
				break;

			case 'H':
				write( outfh, "cf\x1ah", 4);
				Outlen = huff_compress(infh, outfh);
				break;

			default:
				close (infh);
				close (outfh);
				Error(BADOPTION, FATAL, NULL);
			}
		}
	else{
		write( outfh, "cf\x1al", 4);
		Outlen = lzw_compress(infh, outfh);
		}

	if(Outlen==0)
			Error(NOMEM, FATAL, NULL);

	printf("\nCompressed file is %lu bytes long.\n", Outlen + 4);

	/* close files */
	close(infh);
	close(outfh);

	exit(0);
}


