/*
** Support routines for compression demo.
** Use with huff.asm and lzw.asm, discussed in the March, 1991 issue of BYTE.
**
** To build the executable with Turbo:
** tcc -c -mC cshell.c
** tcc -c -mC eshell.c
** tcc -c -mC csubs.c
** tasm huff.asm
** tasm lzw.asm
** tlink c0c cshell csubs lzw huff, crumple.exe,, emu.lib mathc.lib cc.lib
** tlink c0c eshell csubs lzw huff, uncrumple.exe,, emu.lib mathc.lib cc.lib
** These steps will build two files, crumple.exe and uncrumpl.exe, the
** compressor and decompressor.
*/

#include <stdio.h>
#include <fcntl.h>
#include <io.h>
#include <errno.h>
#include <string.h>
#include <sys/stat.h>
#include <stdlib.h>

#include "crumple.h"

/**********************************************************
*	Error(int type, int isfatal, char * string)
*
*  generic error handler
***********************************************************/
void Error(int type, int isfatal, char * string){

	switch(type){
		case C_USAGE:
			printf("usage: crumple inputfile outputfile [-compression type]\n");
			break;
		case E_USAGE:
			printf("usage: uncrumple inputfile outputfile\n");
			break;
		case FILEOPEN:
			printf("Error- Can't open file: %s\n", string);
			break;
		case NOMATCH:
			printf("Error- No files match: %s\n", string);
			break;
		case CANTCREATE:
			printf("Error- Can't create file: %s.\n", string);
			break;
		case FILEREAD:
			printf("Fatal error reading file %s \n", string);
			break;
		case FILEWRITE:
			printf("Fatal error writing file %s\n", string);
			break;
		case NOMEM:
			printf("Fatal error - Insufficient memory\n", string);
			break;
		case BADOPTION:
			printf("Unrecognized option switch: use h, l, or r only\n", string);
			break;
		case BADFORMAT:
			printf("Unexepected file format - file %s\n", string);
			break;
		}
	if(isfatal)
		exit(type);
	return;
}

/**********************************************************
*	open_outfile(char* filespec)
*
* open outputfile. does overwrite check
***********************************************************/
int open_outfile(char* filespec){

int fh;
int i;

	fh=open(filespec,O_CREAT|O_EXCL|O_BINARY,S_IWRITE);
	if(fh==-1){
		if(errno==EEXIST){
			if(!ask("File exists. Overwrite?"))
				return -1;
			fh=open(filespec,O_TRUNC|O_BINARY|O_WRONLY,S_IWRITE);
			if (fh==-1){
				Error(CANTCREATE,NOTFATAL,filespec);
				return -1;
				}
			}
		else{
			Error(CANTCREATE,NOTFATAL,filespec);
			return -1;
			}
		}

	return fh;

}

/**********************************************************
*	ask(char * string)
*
* generic user information routine. Asks yes/no questions.
* one for yes, zero for no.
***********************************************************/
int ask(char * string)
{
int ch, ch0;
	printf("%s [y/n]", string);
	while(1){
		ch0=getch();
		ch=toupper(ch0);
		if(ch=='Y' || ch=='N'){
			printf(" %c\n", ch0);
			return (ch=='Y');
			}
		}
}

