/* 
** shell for testing compression programs.
** BYTE, 11/90
** S. Apiki
** Use with huff.asm and lzw.asm, discussed in the March, 1991 issue of BYTE.
**
** To build the executable with Turbo:
** tcc -c -mC cshell.c
** tcc -c -mC eshell.c
** tcc -c -mC csubs.c
** tasm huff.asm
** tasm lzw.asm
** tlink c0c cshell csubs lzw huff, crumple.exe,, emu.lib mathc.lib cc.lib
** tlink c0c eshell csubs lzw huff, uncrumple.exe,, emu.lib mathc.lib cc.lib
** These steps will build two files, crumple.exe and uncrumpl.exe, the
** compressor and decompressor.
*/

#define DOS

#include <stdio.h>
#include <fcntl.h>
#include <io.h>
#include <errno.h>
#include <string.h>
#include <sys/stat.h>
#include <stdlib.h>

#include "crumple.h"

/* support functions are in csubs.c */
void Error(int type, int isfatal, char * string);/* error handler */
int open_outfile(char* filespec);

/*
* Main program begins here.
*
* Shell takes an input file, an output file, and a duplicate file name.
*
*/

void main(int argc, char** argv)
{

char header[4];
int infh, outfh, dupfh;
unsigned long outlen;

	/* open input file */
	if(argc < 3)
		Error(E_USAGE, FATAL, NULL);
	infh=open(argv[1], O_BINARY|O_RDONLY);
	if(infh == -1)
		Error(FILEOPEN, FATAL, argv[1]);

	/* open output file */
	outfh=open_outfile(argv[2]);
	if (outfh == -1){
		close(infh);
		Error(CANTCREATE, FATAL, argv[2]);
		}

	/* read header */
	read(infh, header, 4);
	if (strncmp(header, "cf\x1a", 3)){
		close(infh);
		close(outfh);
		Error(BADFORMAT, FATAL, argv[1]);
		}

	switch(header[3]){

		case 'l':
			outlen = lzw_expand(infh, outfh);
			break;

		case 'h':
			outlen = huff_expand(infh, outfh);
			break;

		default:
			close(infh);
			close(outfh);
			Error(BADFORMAT, FATAL, argv[1]);
			}

	if (outlen==0)
		Error(NOMEM, FATAL, NULL);

	printf("\nExpanded file is %lu bytes long.\n", outlen);

	/* close files */
	close(infh);
	close(outfh);

	exit(0);

}

